open Thread
open Event

external register_cb: (unit -> unit) -> unit = "register_cb"
external run_cb: unit -> unit = "run_cb"
external run_cb_segfault: unit -> unit = "run_cb_segfault"

type evt = Run | Run_segfault | Finish

let thr_main ch () =
	let running = ref true in
	while !running do
		match (sync (receive ch)) with
		| Run -> run_cb ()
		| Run_segfault -> run_cb_segfault ()
		| Finish -> running := false
	done

let create () =
	let ch = new_channel () in
	let th = Thread.create (thr_main ch) () in
	(th, ch)

let notify (th, ch) data =
	let thr_main ch data () = sync (send ch data) in
	Thread.create (thr_main ch data) ()

let join (th, ch) = Thread.join th

let print () = Printf.printf "hello, world\n"; flush stdout

let main () =
	let thr = create () in
	register_cb print;
	notify thr Run;
	(* Segmentation Fault *)
	notify thr Run_segfault;
	notify thr Finish;
	join thr

let _ = Printexc.catch main ()

